"""
    startResultsContainer(auth_code::String) -> Dict

Starts the results container using the provided authentication code.

# Arguments
- `auth_code::String`: The authentication code required to access the container registry.

# Returns
- `Dict`: A dictionary containing the configuration and status of the started results container.

# Example
> results_container = startResultsContainer("your_auth_code_here")

This function retrieves the configuration for the results container based on the provided authentication code, then starts the container and returns its configuration and status.
"""
function startResultsContainer(auth_code::String)
    results_container = get_results_container(auth_code)
    return results_container
end
"""
    saveResultsContainer(auth_code::String)

Saves the current state of the results container to an image for later use or distribution.

# Arguments
- `auth_code::String`: The authentication code required to access the container registry and perform operations.

# Example
> saveResultsContainer("your_auth_code_here")

This function stops the results container, commits its current state to an image, and pushes this image to a registry using the provided authentication code.
"""
function saveResultsContainer(auth_code::String)
    stop_container_update_image(auth_code)  
end
"""
    stopResultsContainer(auth_code::String)

Stops the results container identified by the authentication code.

# Arguments
- `auth_code::String`: The authentication code required to access and perform operations on the container.

# Example
> stopResultsContainer("your_auth_code_here")

This function stops the running results container using the provided authentication code.
"""
function stopResultsContainer(auth_code::String)
    stop_container("fact_results", auth_token=auth_code)
end
"""
    startDataContainers(services::Vector{String}, auth_code::String)

Starts data containers for the services listed in the `services` argument using the provided authentication code.

# Arguments
- `services::Vector{String}`: A vector of service names for which data containers should be started.
- `auth_code::String`: The authentication code required to access the container registry and perform operations.

# Example
> startDataContainers(["service1", "service2"], "your_auth_code_here")

This function iterates over the given services, retrieves their respective container configurations, and starts each container using the provided authentication code.
"""
function startDataContainers(services::Vector{String}, auth_code::String)
    for service in services
        lower_case_service=lowercase(service)
        container_config = get_data_container_config(lower_case_service)
        create_container(container_config, lower_case_service, auth_token=auth_code)
    end
end
"""
    stopDataContainers(services::Vector{String}, auth_code::String)

Stops data containers for the services listed in the `services` argument using the provided authentication code.

# Arguments
- `services::Vector{String}`: A vector of service names for which data containers should be stopped.
- `auth_code::String`: The authentication code required to access and perform operations on the container.

# Example
> stopDataContainers(["service1", "service2"], "your_auth_code_here")

This function iterates over the given services and stops each corresponding data container using the provided authentication code.
"""
function stopDataContainers(services::Vector{String}, auth_code::String)
    for service in services
        lower_case_service=lowercase(service)
        stop_container(lower_case_service, auth_token=auth_code)
    end

end
"""
    collectServices(tasks::Dict) -> Vector{String}

Collects and returns a list of unique service names from the provided tasks dictionary.

# Arguments
- `tasks::Dict`: A dictionary containing task configurations, each with a service name.

# Returns
- `Vector{String}`: A vector of unique service names extracted from the tasks.

# Example
> services = collectServices(tasks)

This function parses the tasks dictionary to extract the service names associated with each task, removes duplicates, and returns the list of unique service names for further processing.
"""
function collectServices(tasks::Dict)
    services = String[]  # Initialize an empty array to store service values

    for (task_name, task_info) in tasks
        push!(services, task_info["service"])  # Collect each service value
    end

    return unique(services)  # Remove duplicates and return the result
end
