"""
    Module FACT_workflow_manager

The `FACT_workflow_manager` module is designed to facilitate the execution of tasks defined in workflow files, particularly in the context of data analysis and processing. It leverages Julia's capabilities for multithreading, data manipulation, and database interaction, as well as Docker container management for isolated execution environments. The module integrates functionalities for parsing workflow definitions, managing Docker containers, executing tasks based on their dependencies, and handling the input/output of results.

## Dependencies
- `Base.Threads`: For utilizing multithreading capabilities to improve performance and manage concurrent executions.
- `TOML`: To parse workflow definitions from TOML files.
- `MySQL`: To interact with MySQL databases for storing and retrieving task execution results.
- `DataFrames`: For handling and manipulating data within tasks.
- `Random`, `Dates`: Utilized for generating random data and managing date/time operations respectively.
- `FACTResultsIO`, `FACTDataReader`, `JuliaDockerClient`: Custom modules or packages for managing results I/O, reading data specific to the FACT framework, and interacting with Docker containers.

## Key Components
- `TOMLParser.jl`: Contains functions for parsing workflow definitions from TOML formatted files.
- `Containers.jl`: Includes utilities for managing Docker containers, such as starting, stopping, and interacting with containers.
- `Task.jl`: Defines the structure and execution logic for tasks within the workflow, including parameter processing and task execution.
- `Runner.jl`: Implements the logic for running the workflow, managing the execution order of tasks based on dependencies, and orchestrating the overall workflow execution.

## Exported Functions
- `run_file`: The primary interface for executing a workflow defined in a TOML file. It takes the path to a workflow file as input and manages the execution of the defined tasks.

# Usage Example
To execute a workflow defined in a TOML file located at `/path/to/workflow.toml`, ensure that the `FACTWorkflowManager` module is included in your project, and then call:

> using FACT_workflow_manager

> run_file("/path/to/workflow.toml")

This will initiate the workflow execution process, handling task dependencies, Docker container management, and results I/O as defined in the workflow file.
"""
module FACT_workflow_manager

    using Base.Threads, TOML, MySQL, DataFrames, Random, Dates
    using FACT_unified_data_IO, FACT_data_API_reader

    include("TOMLParser.jl")
    include("Containers.jl")
    include("Task.jl")
    include("Runner.jl")

    export run_file, process_task_parameters

end
