"""
    run_file(workflowFilePath::String)

Executes a workflow defined in a file specified by `workflowFilePath`. This involves several key steps:
- Parsing the workflow file to extract tasks and global variables.
- Collecting a list of unique services required for the workflow execution.
- Logging in to an image registry to access necessary container images.
- Starting a results container to hold the outcome of the workflow.
- Starting data containers based on the services identified.
- Managing the execution of the workflow, including the execution of tasks and handling of global variables.
- Optionally updating the results container with the final results of the workflow.
- Stopping the results container and any data containers initiated for the workflow.

This function is designed to automate the setup, execution, and teardown of a containerized workflow, making it easier to run complex processes with multiple dependencies.

# Arguments
- `workflowFilePath::String`: The path to the file containing the workflow definition.

# Side Effects
- Logs into an image registry using predefined credentials.
- Starts and stops Docker containers as required by the workflow.
- May update a results container with the output of the workflow execution.

# Example
To run a workflow defined in a file located at `/path/to/workflow.yml`, call:

> run_file("/path/to/workflow.yml")

This will execute the workflow as defined, handling all necessary container operations behind the scenes.
"""
function run_file(workflowFilePath::String)
    # Parse the workflow file
    tasks, globals = parseWorkflow(workflowFilePath)

    # Extract the 'overwrite' key with a default of false
    overwrite = get(globals, "overwrite", false)

    # Ensure 'overwrite' is a boolean
    overwrite = Bool(overwrite)

    # Get a list of unique services we need for the workflow
    unique_services = collectServices(tasks)

    # Login to image registry
    encoded_auth = registry_login("registry.nilu.no", "FACT_token", "glpat-1zG-TQx___xLsPjj2vsG")

    # Start the results container & get a connection to it
    results_container = startResultsContainer(encoded_auth)

    # Start the data containers
    startDataContainers(unique_services, encoded_auth)

    # Execute the workflow
    need_to_update_results = manage_workflow(tasks, globals["platform"], results_container, overwrite)

    # Stop the results container
    if need_to_update_results
        saveResultsContainer(encoded_auth)
    else
        stopResultsContainer(encoded_auth)
    end

    # Stop the data containers
    stopDataContainers(unique_services, encoded_auth)

end

"""
    manage_workflow(tasks::Dict, platform::Dict, results_container::Dict) -> Ref{Bool}

Executes a set of tasks defined in `tasks`, managing their execution based on dependencies and available system resources defined in `platform`. Each task waits for its dependencies to complete and for a semaphore token, ensuring that the number of parallel tasks does not exceed system resource constraints. The function uses a semaphore to manage the concurrency level based on CPU and memory availability.

# Arguments
- `tasks`: A dictionary containing task names as keys and task details as values. Each task detail is a dictionary that may include dependencies and other execution-related information.
- `platform`: A dictionary containing platform-related settings such as available CPUs (`cpus`), available memory (`mem_GB`), and other information that may be useful for replication purposes.
- `results_container`: A dictionary or object representing the results container, which may be used by tasks to store or retrieve results.

# Returns
- `Ref{Bool}`: A reference to a boolean flag indicating whether any task requested the results container to be saved, potentially due to updates or changes made during task execution.

# Example
To execute a workflow, define `tasks`, `platform`, and `results_container` accordingly and call:

> update_needed = manage_workflow(tasks, platform, results_container)

This function ensures that tasks are executed respecting their dependencies and available resources, aiming to maximize efficiency and minimize resource contention.
"""
function manage_workflow(tasks::Dict, platform::Dict, results_container::Dict, overwrite::Bool)
    # Flag to return whether there is a need to save an updated image of FACTResultsIO
    update_condition = Atomic{Bool}(false)

    # Initialize a Dict to hold Channels for each task
    task_channels = Dict{String, Channel{Bool}}()

    for task_name in keys(tasks)
        # For simplicity, use an unbuffered channel
        task_channels[task_name] = Channel{Bool}(1)
    end

    task_handles = []  # Initialize an array to hold task handles

    # Start the tasks logic
    for (task_name, task_info) in tasks
        #println("Spawning task: $task_name")
        t = Threads.@spawn begin
            try
                # Prepare the dependencies information beforehand
                dependencies_info = Dict{String, Dict}()
                for dep in task_info["dependencies"]
                    dependencies_info[dep] = tasks[dep]
                end
                # Check if the task has dependencies and wait for them if it does
                if haskey(task_info, "dependencies") && !isempty(task_info["dependencies"])
                    for dep in task_info["dependencies"]
                        # Wait for the dependency task to signal completion
                        #println("[$task_name] waiting on dependency $dep")
                        take!(task_channels[dep])
                        # put back the signal into the channel because somebody else may need it!!!
                        put!(task_channels[dep],true)
                        #println("[$task_name] dependency $dep completed")
                    end
                end
                # Execute the task
                #println("Executing task: $task_name")
                run_task(task_name, task_info, dependencies_info, results_container, update_condition, overwrite)
                #println("Finished executing task: $task_name")
            catch ex
                println("Error occurred in task $task_name: ", ex)
                println("Stacktrace: ", stacktrace(catch_backtrace()))
            finally
                put!(task_channels[task_name], true)
                #println("Finished executing task: $task_name")
            end
        end
        push!(task_handles, t)  # Collect the task handle
    end
    
    # Wait on each task to complete
    #println("Waiting for all tasks to complete...")
    foreach(wait, task_handles)  # This waits for each task to complete

    println("All tasks completed.")
    return update_condition[]
end
