# Define a global cache dictionary. The key is a tuple of parameters, and the value is the DataFrame result.
# This cache is used to avoid redundant database queries for task dependencies.
const RESULTS_CACHE = Dict{Tuple{String, String}, DataFrame}()

"""
    run_task(task_name::String, task_info::Dict, dependencies_info::Dict, results_container::Dict, update_condition::Ref{Bool}, overwrite::Bool)

Executes a specified task identified by `task_name`, using information provided in `task_info`, and updates a shared condition if the task execution meets a certain condition.
This involves processing task parameters to resolve dependencies, executing the task's main work function, managing database connections for storing or retrieving results, and handling the overwrite behavior based on the `overwrite` flag.

# Arguments
- `task_name::String`: The name of the task to be executed.
- `task_info::Dict`: A dictionary containing information about the task, including the function to call, arguments, and parameters dictionary.
- `dependencies_info::Dict`: A dictionary containing information about other tasks that the current task may depend on.
- `results_container::Dict`: A dictionary containing connection details to the results storage system.
- `update_condition::Ref{Bool}`: A reference to a boolean flag that is updated based on the outcome of the task execution.
- `overwrite::Bool`: A boolean flag indicating whether existing results should be overwritten in the case of a task rerun.

# Side Effects
- Prints the start and completion time of the task execution.
- May update a shared boolean flag (`update_condition`) if the task execution meets a predefined condition.
- Handles database connections, including opening and closing connections to the results storage system.
- Handles the overwrite logic for task execution results based on the `overwrite` flag.
- Prints error messages if any exceptions occur during the task execution.

# Example
Given a task name, its information, dependencies information, results container details, a reference to an update condition, and an overwrite flag:

> run_task("DataProcessing", task_info, dependencies_info, results_container, update_condition, true)

This function will process the task parameters, execute the task, update the condition if applicable, manage database connections throughout the process, and handle overwriting of results based on the `overwrite` flag.
"""

function run_task(task_name::String, task_info::Dict,  dependencies_info::Dict, results_container::Dict, update_condition::Atomic{Bool}, overwrite::Bool)
    start_time = now()  # Capture the start time
    println("Starting task $task_name at $(Dates.format(start_time, "HH:MM:SS"))")

    condition_met = false

    # in case of need process parameters dictionary to get results from dependencies in database
    parameters_dictionary = process_task_parameters(task_info["parameters_dict"], dependencies_info, results_container)

    try
        condition_met = task_work(task_info["function"], task_info["arguments"], parameters_dictionary, results_container, overwrite)
    catch e
        println("An error occurred in task $task_name: $e")
    end

    # Update condition
    if !update_condition[] && condition_met
        atomic_cas!(update_condition, false, true) # CAS: Compare-And-Swap
    end
    
    end_time = now()  # Capture the end time
    println("Completed task $task_name at $(Dates.format(end_time, "HH:MM:SS"))")

    # Calculate and print the duration
    duration = print_duration_in_hms(end_time - start_time)  # Duration as a String interpretation of a Period type
    println("Duration of task $task_name: $duration")

end

"""
    task_work(functionName::String, arguments::String, parametersDict::Dict, rresults_container::Dict, overwrite::Bool) -> Bool

Executes a specified function with arguments and parameters, interacting with a database through a given connection. If the `overwrite` flag is set to `true`, existing data corresponding to the function's execution will be overwritten in the database. Otherwise, if the data already exists and `overwrite` is `false`, the function will not execute again to avoid duplicating results. If no existing data is found or `overwrite` is `true`, the function is dynamically called with the provided arguments and parameters, and the result is stored in the database.

# Arguments
- `functionName::String`: The name of the function to be executed.
- `arguments::String`: A string representation of the arguments to be passed to the function.
- `parametersDict::Dict`: A dictionary of parameters to be passed to the function.
- `results_container::Dict`: The informationm to activate a database connection for storing or retrieving the function's results.
- `overwrite::Bool`: A boolean flag indicating whether to overwrite existing data in the database.

# Returns
`Bool`: A boolean value indicating whether the function was executed. Returns `true` if the function was called and executed (with results stored or overwritten in the database), or `false` if the results already existed and `overwrite` was `false`, thus no execution was necessary.

# Example
To execute a function named 'computeStatistics' with arguments and parameters, ensuring results are either stored in or overwritten in a MySQL database depending on the `overwrite` flag:

> task_work("computeStatistics", "argumentString", {"param1" => "value1"}, results_container, true)

This example will overwrite existing results in the database for the given function name, arguments, and parameters. If `overwrite` were `false`, it would store the results only if they did not already exist.
"""
function task_work(functionName::String, arguments::String, parametersDict::Dict, results_container::Dict, overwrite::Bool)
    condition_met = false
    # Connect to the results container
    results_connection = get_connection(results_container)

    does_table_exist = does_table_exist_for_method(results_connection, functionName, arguments, parametersDict)
    
    #println("Does table exist: $does_table_exist for $functionName, $arguments, $parametersDict")

    # Close the connection to the results container
    close_connection(results_connection)

    if !does_table_exist || overwrite
        # Prepare arguments string representation for eval
        args_str = parametersDict |> Dict |> string
        call_str = "$functionName(\"$arguments\", $args_str)"
        #println("Executing $call_str")
        # Convert call string to expression and evaluate it
        expr = Meta.parse(call_str)
        df = eval(expr)

        # Connect to the results container
        results_connection = get_connection(results_container)
        
        if overwrite
            overwrite_table(results_connection, functionName, arguments, parametersDict, df)
        else 
            write_table(results_connection, functionName, arguments, parametersDict, df)
        end
        
        # Close the connection to the results container
        close_connection(results_connection)
        
        condition_met = true
    end

    return condition_met
end

"""
    process_task_parameters(original_dict::Dict, dependencies_info::Dict, results_container::Dict) -> Dict

Processes a dictionary of task parameters to resolve any references to data produced by other tasks. References are identified
by a specific format (__REF__taskName[!, :columnName]) and are replaced with the actual data values from the task outputs.
This function supports dynamic parameter passing between tasks in a workflow, where the output of one task can be used as an
input parameter for another.

# Arguments
- `original_dict::Dict`: The original dictionary containing the task parameters, potentially including references to other tasks' outputs.
- `dependencies_info::Dict`: A dictionary containing information about the tasks and their dependencies.
- `results_container::Dict`: The information to activate a database connection used to fetch data referenced by task outputs.

# Returns
`Dict`: A new dictionary with all references resolved to actual data values. If a reference cannot be resolved due to missing data or incorrect reference format, an error is raised.

# Example
Given a task parameter dictionary with references to outputs of other tasks, this function will resolve those references:

> process_task_parameters(original_parameters, dependencies_info, results_connection)

This will return a new dictionary where all __REF__ references have been replaced with the actual values from the respective tasks' outputs.
"""
function process_task_parameters(original_dict::Dict, dependencies_info::Dict, results_container::Dict)
    # Define the regular expression pattern to match the reference format
    ref_pattern = r"__REF__(\w+)\[!, :(\w+)\]"

    # Initialize a new dictionary to hold the processed parameters
    new_dict = Dict()

    for (key, value) in original_dict
        if occursin("__REF__", string(value))
            # Extract task name and column name from the reference
            match_result = match(ref_pattern, value)

            if match_result !== nothing
                task_name = String(match_result.captures[1])
                column_name = Symbol(match_result.captures[2])

                # Retrieve the DataFrame using the task name
                df = get_task_df(task_name, dependencies_info, results_container)

                # Check if the DataFrame and the column exist
                if df !== nothing && String(column_name) in names(df)
                    # Update the new dictionary with the column value
                    new_dict[key] = df[!, column_name]
                else
                    error("DataFrame or column not found for reference: $value")
                end
            else
                error("Invalid reference format in value: $value")
            end
        else
            # For non-reference values, just copy them to the new dictionary
            new_dict[key] = value
        end
    end

    return new_dict
end

"""
    get_task_df(task_name::String, dependencies_info::Dict, results_connection::MySQL.Connection) -> DataFrame

Retrieves a DataFrame for a given task by executing a database query through a results connection. 
This function looks up the task's details in `dependencies_info` using `task_name` as the key. If the task is found,
it extracts the function name, arguments, and parameters dictionary from the task's definition and then queries the database 
to fetch the corresponding data.

# Arguments
- `task_name::String`: The name of the task for which data is to be fetched.
- `dependencies_info::Dict`: A dictionary containing task definitions, including function names, arguments, and parameters.
- `results_connection::MySQL.Connection`: An active database connection through which the query will be executed.

# Returns
`DataFrame`: A DataFrame containing the results of the query. Returns an empty DataFrame if the task is not found in `dependencies_info`.

# Example
Assuming `dependencies_info` is a dictionary with task definitions and `results_connection` is an established MySQL connection:

> get_task_df("myTask", dependencies_info, results_connection)

This will return a DataFrame with the data for `myTask`, based on the details specified in `dependencies_info`.
"""
function get_task_df(task_name::String, dependencies_info::Dict, results_container::Dict)

    # Check for a cached result first
    cache_key = (task_name, string(hash(dependencies_info)))  # Generate a unique key for the current parameters
    if haskey(RESULTS_CACHE, cache_key)
        #println("Using cached results for task $task_name")
        return RESULTS_CACHE[cache_key]  # Return the cached DataFrame
    end

    # Connect to the results container
    results_connection = get_connection(results_container)

    df = DataFrame()

    if haskey(dependencies_info, task_name)
        task_definition = dependencies_info[task_name]

        # Extract the required information
        functionName = String(get(task_definition, "function", ""))  # Default to empty string if not found
        arguments = String(get(task_definition, "arguments", ""))  # Adjust based on actual key if different
        parametersDict = Dict(get(task_definition, "parameters_dict", Dict()))

        # Assuming get_table is a function that queries the database and returns a DataFrame
        df = get_table(results_connection, functionName, arguments, parametersDict)

        # Cache the result before returning
        RESULTS_CACHE[cache_key] = df
    end

    # Close the connection to the results container
    close_connection(results_connection)

    return df
end

function print_duration_in_hms(duration::Period)
    total_milliseconds = Dates.value(duration)  # Get total milliseconds from the duration
    total_seconds = div(total_milliseconds, 1000)  # Convert milliseconds to seconds
    hours = total_seconds ÷ 3600  # Integer division to get hours
    minutes = (total_seconds % 3600) ÷ 60  # Remaining minutes
    seconds = total_seconds % 60  # Remaining seconds

    # Format the string with leading zeros for minutes and seconds
    return "$(hours)h:$(lpad(minutes, 2, "0"))m:$(lpad(seconds, 2, "0"))s"
end